from typing import Any, List, Optional

from fastapi import APIRouter, Depends, HTTPException, status
from sqlalchemy.orm import Session

from app import crud, schemas
from app.db.session import get_db

router = APIRouter()


@router.get("/", response_model=List[schemas.Product])
def read_products(
    db: Session = Depends(get_db),
    skip: int = 0,
    limit: int = 100,
    name: Optional[str] = None,
    category_id: Optional[str] = None,
    supplier_id: Optional[str] = None,
    is_active: Optional[bool] = None,
    low_stock: Optional[bool] = None,
) -> Any:
    """
    Retrieve products with optional filtering.
    """
    filters = {
        "name": name,
        "category_id": category_id,
        "supplier_id": supplier_id,
        "is_active": is_active,
        "low_stock": low_stock,
    }

    # Remove None values
    filters = {k: v for k, v in filters.items() if v is not None}

    products = crud.product.get_multi(db, skip=skip, limit=limit, filters=filters)
    return products


@router.post("/", response_model=schemas.Product, status_code=status.HTTP_201_CREATED)
def create_product(
    *,
    db: Session = Depends(get_db),
    product_in: schemas.ProductCreate,
) -> Any:
    """
    Create new product.
    """
    # Check if product with the same SKU already exists
    if product_in.sku:
        product = crud.product.get_by_sku(db, sku=product_in.sku)
        if product:
            raise HTTPException(
                status_code=status.HTTP_400_BAD_REQUEST,
                detail="Product with this SKU already exists.",
            )

    # Check if product with the same barcode already exists
    if product_in.barcode:
        product = crud.product.get_by_barcode(db, barcode=product_in.barcode)
        if product:
            raise HTTPException(
                status_code=status.HTTP_400_BAD_REQUEST,
                detail="Product with this barcode already exists.",
            )

    product = crud.product.create(db, obj_in=product_in)
    return product


@router.get("/{product_id}", response_model=schemas.ProductWithDetails)
def read_product(
    *,
    db: Session = Depends(get_db),
    product_id: str,
) -> Any:
    """
    Get product by ID.
    """
    product = crud.product.get(db, product_id=product_id)
    if not product:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail="Product not found",
        )
    return product


@router.put("/{product_id}", response_model=schemas.Product)
def update_product(
    *,
    db: Session = Depends(get_db),
    product_id: str,
    product_in: schemas.ProductUpdate,
) -> Any:
    """
    Update a product.
    """
    product = crud.product.get(db, product_id=product_id)
    if not product:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail="Product not found",
        )

    # Check if trying to update to existing SKU
    if product_in.sku and product_in.sku != product.sku:
        existing_product = crud.product.get_by_sku(db, sku=product_in.sku)
        if existing_product and existing_product.id != product_id:
            raise HTTPException(
                status_code=status.HTTP_400_BAD_REQUEST,
                detail="Product with this SKU already exists",
            )

    # Check if trying to update to existing barcode
    if product_in.barcode and product_in.barcode != product.barcode:
        existing_product = crud.product.get_by_barcode(db, barcode=product_in.barcode)
        if existing_product and existing_product.id != product_id:
            raise HTTPException(
                status_code=status.HTTP_400_BAD_REQUEST,
                detail="Product with this barcode already exists",
            )

    product = crud.product.update(db, db_obj=product, obj_in=product_in)
    return product


@router.delete("/{product_id}", status_code=status.HTTP_204_NO_CONTENT, response_model=None)
def delete_product(
    *,
    db: Session = Depends(get_db),
    product_id: str,
) -> Any:
    """
    Delete a product.
    """
    product = crud.product.get(db, product_id=product_id)
    if not product:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail="Product not found",
        )

    # Check if product has inventory movements before deleting
    if product.inventory_movements and len(product.inventory_movements) > 0:
        raise HTTPException(
            status_code=status.HTTP_400_BAD_REQUEST,
            detail="Cannot delete product with inventory movements",
        )

    crud.product.remove(db, product_id=product_id)
    return None


@router.patch("/{product_id}/stock", response_model=schemas.Product)
def update_product_stock(
    *,
    db: Session = Depends(get_db),
    product_id: str,
    stock_update: schemas.StockUpdate,
) -> Any:
    """
    Manually update product stock.
    """
    product = crud.product.get(db, product_id=product_id)
    if not product:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail="Product not found",
        )

    # Create inventory movement record for audit
    movement_in = schemas.InventoryMovementCreate(
        product_id=product_id,
        quantity=stock_update.quantity,
        type=schemas.MovementTypeEnum.ADJUSTMENT,
        notes=stock_update.notes or "Manual stock adjustment",
    )

    # This would be inside a transaction in a real-world scenario
    # Create the movement record and update stock
    crud.inventory.create(db, obj_in=movement_in)
    product = crud.product.update_stock(db, db_obj=product, quantity=stock_update.quantity)

    return product
